/**************************************************************

   Program Title:  Wasp AVR Monitor  Data Logger
   
   Program Name:  Wasp_Monitor.c	  Version: 0.94
   
   Author:  SS	  					  Date:  Dec 15, 2010
   
   Compiled with ImageCraft ICCAVR Standard V7.13A.
   
   This verison is designed for the ATmega644 processor:
   		64K Flash, 2K EEPROM, 4K RAM
		
   If the Serial Flash and/or Accelerometer are installed then a 10MHz
   crystal should be installed in place of the 20MHz crystal as the 20MHz
   crystal requires the processor be powered by 5V which the serial flash
   and Accelerometer can't handle.
		
   Fuse bit:  FF D9 E2  - Disable JTAG - use internal OSC
   Fuse bit:  FF D9 EE  - Disable JTAG - use external 10MHz Xtal
   Fuse bit:  FF D9 E7  - Disable JTAG - use external 20MHz Xtal
	
   This program will not run properly if DIV8 fuse bit is set (programmed)
   and JTAG pins are enabled - this is the chip default.
		
   The Wasp Monitor is a set of functions that configure Wasp AVR internal
   peripherals for interrupt driven operation.  Custom applications can easily
   be built on top to these basic functions.  Included is a complete data 
   acquistion controller with user preferences loaded into EEPROM.  The Monitor
   communicates with the user via an RS-232 port using the CDP.   The AVR is 
   a smart powerful 8 bit RISC processor that can be easily programmed in C
   with on chip TWI, SPI and analog and digital IO.  Functions are included 
   to read/write data blocks to the on board Serial Flash.  On board sensors
   (if included) include a dual axis 2G accelerometer, temperature and light
   sensor and battery voltage monitor.
   
   Sensors:
   		AD0 - Accelerometer channel Y
		AD1 - Accelerometer channel Z
		AD5 - Accelerometer channel X
		AD2 - Battery Voltage
		AD3 - Temperature 
		AD4 - Light level
		
   Key functions:
   	   SPI Master with read/write function to serial flash
	   TWI 2-Wire Master Interrupt Controller
	   Real Time Clock setup with four configurable software timers
	   Full duplex interrupt drive USART with XON/XOFF flow control
	   1msec timer with four configurable software times
	   Analog controller setup
	   Digital port configuration
	   Accelerometer reader
	   Temperature reader
	   Light level reader
	   IntelHex32 download/upload for EEPROM, AVRFLASH and Serial Flash
	   Auto clock calibration
   
   Copyright 2004-2010, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2010, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.94  2011/11/05 SS
 *    Fixed data logging routines and added support for 10DOFW
 * Revision 0.93  2010/11/05 DL
 *    Added TWI functions to ADS1100 ADC chip
 * Revision 0.92  2010/06/11 RS
 *    Changed to support ImageCraft ICCAVR Standard V7.20.compiler
 * Revision 0.91  2010/03/04 SS
 *    DC200 Support added
 * Revision 0.90  2009/06/20 SS
 *    Serial Flash data logger support for 3-axis rate gyro added
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
#include <iom644v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
//#include <AVRdef.h>
#include "Wasp_Uart.h"
#include "Wasp_Defines.h"
#include "Wasp_Extern.h"

//  Variable declarations
unsigned int bytesonline;
unsigned char programmode,crc,intelbinary[34];  // length can vary
unsigned char hexbyte[3],hexaddress[5],intelrecordcommand;
unsigned char seconds,minutes,hours,days,rtcclockcnt,loggingmode,data2log;
unsigned char flashtype0,flashtype1,flashpageloaded,flashpageboundarytraversed;
unsigned char flashprogrammingactive,flashdatainbuffer,ascii[16],bufferidptr;
unsigned char version[30],i2cmode,armpd2,smartlcdstate,digitalcompassstate,ADSstate;
unsigned char TWI_slaveAddress,TWI_masterAddress,BMPstate,deviceid,dataformat;
unsigned char deviceidtext[21],terminationmode,rtccnt;

unsigned int flashpagesize0,flashpagesize1,pagecnt,timeoutperiod,elapsedtime;
unsigned int intelsuperaddress,oldpage,oldbyteoffset,totalrxcnt;
unsigned int rtccountdown1,rtccountdown2,rtccountdown3,rtccountdown4;
unsigned int mseccountdown1,mseccountdown2,mseccountdown3,flashcountdown;

unsigned char messageBuf[TWIMESSAGELENGTH];

// BMP085 Parameters
int  bmpac1,bmpac2,bmpac3,bmpb1,bmpb2,bmpmb,bmpmc,bmpmd;
unsigned int bmpac4,bmpac5,bmpac6;

// L3G4200D parameters
unsigned char dof10wmode,l3g4200dstate;

// HMC5883 parameters
unsigned char hmc5883state;

unsigned int totalbytecnt,xoncnt,xoffcnt;
unsigned char frameerror,dataoverrun,defaultaccelerometerrange,logoutputmode;
int  cursorlocation,defaultlograte;
int xgmax,xgmin,xgmid,ygmax,ygmin,ygmid,zgmax,zgmin,zgmid;
long flashaddress,inteladdress,oldinteladdress;

int _textmode;

/* Static Variables */
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars,sendxoff;
char  rxcnt,checkflag,rxcntmax;
unsigned char  instrg[8],vlow,vhigh;

unsigned char FlashAddress[3];
const char signon1[] = "\n\r\n\rWasp ATmega644 Monitor Data Logger V0.94\n\r";
const char signon2[] = "(c) Copyright 2004-2011,  SOC Robotics, Inc.\n\r";
const char signon3[] = "\n\r   D[E|A|F]  - Display AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon4[] = "   P[E|A|F]  - Program AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon5[] = "   L   - Enter log sensor data mode\n\r";
const char signon6[] = "   G   - Measure gravity using accelerometer\n\r";
const char signon62[] = "   D   - Return digital compass reading\n\r";
const char signon65[] = "   Rn  - Measure rate gyro 1,2 or 3 axis\n\r";
const char signon66[] = "   RG  - Measure gravity and 3-axis gyro\n\r";
const char signon67[] = "   M   - Measure all analog inputs and output statistics\n\r";
const char signon7[] = "   V   - Version\n\r";
const char signon8[] = "   C   - Digital Compass heading (deg)\n\r";
const char signon83[] = "   T   - Get/Set Time\n\r";
const char signon85[] = "   A   - Read all Analog channels\n\r";
const char signon9[] = "   S   - Signon message\n\r";
const char signon10[] = "   Z   - Read ADS1100\n\r";

const char displaymessage1[] = "Enter start address and length: ";
const char displaymessage2[] = "\n\rDisplay AVR Flash from 0x0054 to 254\n\r";
const char displaymessage3[] = "\n\rDisplay SPI Flash from 0 to 100\n\r";
const char displaymessage33[] = "\n\rNo mode selected\n\r";

const char progmessage[] = "Wasp Monitor Version: ";

const char displaymessage4[] = "\n\rLoad EEPROM from address ";
const char displaymessage5[] = " length ";
const char displaymessage6[] = "\n\rLoad SPI Flash from address ";

const char loadmessage1[] = "\n\rLoad EEPROM\n\r";
const char loadmessage2[] = "\n\rLoad SPI Flash\n\r";
const char loadmessage3[] = "\n\rLoad AVR Flash\n\r";

// Function declarations
unsigned char EEPROMwrite(int,unsigned char);
unsigned char EEPROMread(int);
int printf(CONST char *, ...);
int sprintf(char *, CONST char *, ...);
int scanf(CONST char *, ...);
int sscanf(char *, CONST char *, ...);

// Initial MMA7260 3-Axis accelerometer
void Init_3Axis_Accelerometer(char mode)
{
 	 // Set direction bits
	 //   SG2 - PC5, SG1 - PC4, SLP - PC3
	 DDRC |= 0x3C;
 	 // Enable Accelerometer
	 SetBit(PORTC,PORTC3);
	 
	 if(mode==G15MODE) { ClrBit(PORTC,PORTC5); ClrBit(PORTC,PORTC4); }
	 else if(mode==G2MODE) { ClrBit(PORTC,PORTC5); SetBit(PORTC,PORTC4); }
	 else if(mode==G4MODE) { SetBit(PORTC,PORTC5); ClrBit(PORTC,PORTC4); }
	 else if(mode==G6MODE) { SetBit(PORTC,PORTC5); SetBit(PORTC,PORTC4); }
}

void Display_Gravity_UART(char mode)
{
    unsigned char firstmeasurement;
	unsigned char inchar,firstrun=YES;
    int i,xmin,xmax,samples,delaytime=150;
	int xgravity,ygravity,zgravity;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
	float xgrav,ygrav,zgrav,xamax,yamax,zamax,xamin,yamin,zamin;
	
	Init_3Axis_Accelerometer(defaultaccelerometerrange);
	if(mode==GONLY) printf("\n\r3-Axis Acceleration [e-exit,g-new range,r-raw,p-processed,c-calibrate]\n\r");
	if(defaultaccelerometerrange==G15MODE) printf("Range is 1.5G\n\r");
	else if(defaultaccelerometerrange==G2MODE) printf("Range is 2G\n\r");
	else if(defaultaccelerometerrange==G4MODE) printf("Range is 4G\n\r");
	else if(defaultaccelerometerrange==G6MODE) printf("Range is 6G\n\r");
	printf("   X      Y      Z        Xmin   Ymin   Zmin     Xmax   Ymax   Zmax\n\r");
	xamin = yamin = zamin = 9.0;
	xamax = yamax = zamax = -9.0;
	
	while((inchar=getcharnw())!='e') {
	    // If range switch do this first before sampling data
		if(inchar==CHANGERANGE) {
		    printf("\n\rEnter new G range [1-1.5,2,4,6]: ");
			defaultaccelerometerrange = getchar();
			printf("%c\n\r",defaultaccelerometerrange);
			Init_3Axis_Accelerometer(defaultaccelerometerrange);
			xamin = yamin = zamin = 9.0;
			xamax = yamax = zamax = -9.0;
			Delaymsec(400);
			SaveEepromPreferences();
		    }
			
		// Now get the new data
		xgravity = ygravity = zgravity = 0;	
		samples = 3;
		for(i=0; i<samples; i++) {
			xgravity += AnalogCh(5,EXTERNALREF);
			ygravity += AnalogCh(0,EXTERNALREF);
			zgravity += AnalogCh(1,EXTERNALREF);
			}

		xgravity = (xgravity/samples);
		ygravity = (ygravity/samples);
		zgravity = (zgravity/samples);
		
		if(inchar==RAW) dataformat = RAW;
		else if(inchar==PROCESSED) dataformat = PROCESSED;
		else if(inchar==CHANGESPEED) {
		    printf("Enter new delay speed: ");
			delaytime = readint();
		    }
		else if(inchar==CALIBRATE) {
		    printf("\n\rSelect manual calibration or automatic [m,a]:");
			if(getchar()=='m'){
			    putchar('m');
			    printf("\n\rCurrent calibration values:\n\r xgmin=%d xgmax=%d xgmid=%d\n\r",
				          xgmin,xgmax,xgmid);
				printf(" ygmin=%d ygmax=%d ygmid=%d\n\r",ygmin,ygmax,ygmid);
				printf(" zgmin=%d zgmax=%d zgmid=%d\n\r",zgmin,zgmax,zgmid);
			    printf("m\n\rEnter xgmin value: ");
				xgmin = readint();
				printf("\n\rEnter xgmax value: ");
				xgmax = readint();
				printf("\n\rEnter xgmid value: ");
				xgmid = readint();
			    printf("m\n\rEnter ygmin value: ");
				ygmin = readint();
				printf("\n\rEnter ygmax value: ");
				ygmax = readint();
				printf("\n\rEnter ygmid value: ");
				ygmid = readint();
			    printf("m\n\rEnter zgmin value: ");
				zgmin = readint();
				printf("\n\rEnter zgmax value: ");
				zgmax = readint();
				printf("\n\rEnter zgmid value: ");
				zgmid = readint();
				printf("\n\r");
				SaveEepromPreferences();
				}
			else {
			    printf("Begin calibration of 3-axis accelerometer\n\r");
				printf("Lay Wasp on flat surface\n\r");
				Delaymsec(1000);
				printf("Sweep Wasp +-270degree slowly through X-axis\n\r");

				Delaymsec(1000);
				printf("Sweep Wasp +-270degree slowly through Y-axis\n\r");

				Delaymsec(1000);
				printf("Sweep Wasp +-270degree slowly through Z-axis\n\r");

				Delaymsec(1000);
				}
			
		    }
		
		//printf("X Y Z accel = ");
		if(dataformat==RAW) printf("%5d %4d %4d     \r",
		      xgravity,ygravity,zgravity);
		else {
		    if(xgravity>xgmid) xgrav = (float)(xgravity-xgmid)/(float)(xgmax-xgmid);
			else xgrav = (float)(xgravity-xgmid)/(float)(xgmid-xgmin);
			
			if(ygravity>ygmid) ygrav = (float)(ygravity-ygmid)/(float)(ygmax-ygmid);
			else ygrav = (float)(ygravity-ygmid)/(float)(ygmid-ygmin);
			
			if(zgravity>zgmid) zgrav = (float)(zgravity-zgmid)/(float)(zgmax-zgmid);
			else zgrav = (float)(zgravity-zgmid)/(float)(zgmid-zgmin);
			
			if(defaultaccelerometerrange==G2MODE) {
			    xgrav *= 2.0/1.5;
				ygrav *= 2.0/1.5;
				zgrav *= 2.0/1.5;
				}
			else if(defaultaccelerometerrange==G4MODE) {
			    xgrav *= 4.0/1.5;
				ygrav *= 4.0/1.5;
				zgrav *= 4.0/1.5;
				}
			else if(defaultaccelerometerrange==G6MODE) {
			    xgrav *= 6.0/1.5;
				ygrav *= 6.0/1.5;
				zgrav *= 6.0/1.5;
				}
			
			if(xgrav>xamax)  xamax = xgrav;
			if(ygrav>yamax)  yamax = ygrav;
			if(zgrav>zamax)  zamax = zgrav;
			if(xgrav<xamin)  xamin = xgrav;
			if(ygrav<yamin)  yamin = ygrav;
			if(zgrav<zamin)  zamin = zgrav;			
			
			printf("%# 6.3f %# 6.3f %# 6.3f ",xgrav,ygrav,zgrav);
			if(1) {
			    printf("   %# 6.3f %# 6.3f %# 6.3f ",xamin,yamin,zamin);
				printf("  %# 6.3f %# 6.3f %# 6.3f ",xamax,yamax,zamax);
				}
			printf("\r");
			} 
		Delaymsec(delaytime);
		}
	printf("\n\r-");
}

// Rate gyro change is 0.67mv per degree or +-67mv
//   Default output is 1.350V - 1.35/3.3 = 0.409
//   0.409*999 = 418
void Display_RateGyro_UART(unsigned char numaxis)
{
	unsigned char inchar;
    int i,gyrorate,gyrorate2,gyrorate3,gyromin,gyromax;
	int xgravity,ygravity,zgravity;
	
	if(numaxis=='1')  {
	    printf("\n\rRate Gyro Output - one axis\n\r");
		printf("        rate  min   max   diff\n\r");
	    gyromin = gyromax = gyrorate = AnalogCh(4,EXTERNALREF);
	    gyromin = gyromax = gyrorate = AnalogCh(4,EXTERNALREF);
	
	    while(!(inchar=getcharnw())) {
	        gyrorate = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    if(gyrorate<gyromin) gyromin = gyrorate;
		    if(gyrorate>gyromax) gyromax = gyrorate;

		    printf("gyro = %4d  %4d  %4d  %4d  \r",gyrorate,gyromin,gyromax,gyromax-gyromin);
		    }
		}
	else if(numaxis=='2') {
		printf("\n\rRate Gyro Output - two axis\n\r");
	
	    while(!(inchar=getcharnw())) {
	        gyrorate = gyrorate2 = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    gyrorate2 += AnalogCh(6,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    gyrorate2 = (gyrorate2/4);

		    printf("gyro = %4d  %4d  \r",gyrorate,gyrorate2);
		    }
	    }
	else if(numaxis=='3') {
		printf("\n\rRate Gyro Output - three axis\n\r");
	
	    while(!(inchar=getcharnw())) {
	        gyrorate = gyrorate2 = gyrorate3 = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    gyrorate2 += AnalogCh(6,EXTERNALREF);
			    gyrorate3 += AnalogCh(7,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    gyrorate2 = (gyrorate2/4);
		    gyrorate3 = (gyrorate3/4);

		    printf("gyro = %4d  %4d  %4d  \r",gyrorate,gyrorate2,gyrorate3);
		    }
	    }
	else if(numaxis=='g') {
		printf("\n\r3-Axis Acceleration + 3-Axis Rate Gyro\n\r");
	
		Init_3Axis_Accelerometer(G15MODE);
	    while(!(inchar=getcharnw())) {
			xgravity = ygravity = zgravity = 0;
		    for(i=0; i<4; i++) {
			    xgravity += AnalogCh(5,EXTERNALREF);
			    ygravity += AnalogCh(0,EXTERNALREF);
			    zgravity += AnalogCh(1,EXTERNALREF);
			    }

		    xgravity = (xgravity/4);
		    ygravity = (ygravity/4);
		    zgravity = (zgravity/4);

		    printf("X Y Z accel = %4d  %4d  %4d ",xgravity,ygravity,zgravity);
			
	        gyrorate = gyrorate2 = gyrorate3 = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    gyrorate2 += AnalogCh(6,EXTERNALREF);
			    gyrorate3 += AnalogCh(7,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    gyrorate2 = (gyrorate2/4);
		    gyrorate3 = (gyrorate3/4);

		    printf("gyro = %4d  %4d  %4d  \r",gyrorate,gyrorate2,gyrorate3);
					xgravity = ygravity = zgravity = 0;	
		    }
	    }	
		printf("\n\r-");
}

char FlashBlankCheck()
{
 	 char byte;
	 
	 Release_SPI();
	 Init_SPI_Master();
	 FlashChipSelect(LOW);  
	 SPI_StartReadFlashPage(0);
	 byte = SPI_MasterReceive();
	 FlashChipSelect(HIGH); 
	 Release_SPI(); 
	 if(byte!=0xFF) return YES;
     else return NO;
}

const char devicemes1[] = "Left Arm Sensor ";
const char devicemes2[] = "Right Arm Sensor ";
const char devicemes3[] = "Upper Spine Sensor ";
const char devicemes4[] = "Lower Spine Sensor ";
const char devicemes5[] = "Sled Sensor ";

void DisplaySensorID2()
{
 	 printf("%s ",deviceidtext);

// 	 if(deviceid==LEFTARM) putfs(devicemes1);
// 	 else if(deviceid==RIGHTARM) putfs(devicemes2);
// 	 else if(deviceid==UPPERSPINE) putfs(devicemes3);
//	 else if(deviceid==LOWERSPINE) putfs(devicemes4);
// 	 else if(deviceid==SLED) putfs(devicemes5);
}

void DisplaySensorID(unsigned byte)
{
 	 printf("%s ",deviceidtext);
//	 if((byte&0xf0)==0x10) putfs(devicemes1);
//	 else if((byte&0xf0)==0x20) putfs(devicemes2);
//	 else if((byte&0xf0)==0x30) putfs(devicemes3);
//	 else if((byte&0xf0)==0x40) putfs(devicemes4);
//	 else if((byte&0xf0)==0x50) putfs(devicemes5);
}

void DisplayAccelerometerRange(unsigned byte)
{
	 if((byte&0x0c)==0x00) printf("1.5G ");
	 else if((byte&0x0c)==0x04) printf("2G ");
	 else if((byte&0x0c)==0x08) printf("4G ");
	 else if((byte&0x0c)==0x0c) printf("6G ");
}

unsigned char Convertdata2log(unsigned char loggeddata)
{
 	 if((loggeddata&0x03)==0x00)  return 'a';
	 else if((loggeddata&0x03)==0x01)  return 'g';
	 else if((loggeddata&0x03)==0x02)  return 'b';
	 else if((loggeddata&0x03)==0x03)  return '8';
	 return 'a';
}

float ConvertXgrav(int xaxis)
{
    if(xaxis>xgmid) return (float)(xaxis-xgmid)/(float)(xgmax-xgmid);
	else return (float)(xaxis-xgmid)/(float)(xgmid-xgmin);
}

float ConvertYgrav(int yaxis)
{
	if(yaxis>ygmid) return (float)(yaxis-ygmid)/(float)(ygmax-ygmid);
	else return (float)(yaxis-ygmid)/(float)(ygmid-ygmin);
}
			
float ConvertZgrav(int zaxis)
{
	if(zaxis>zgmid) return (float)(zaxis-zgmid)/(float)(zgmax-zgmid);
	else return (float)(zaxis-zgmid)/(float)(zgmid-zgmin);
}

// Retreive logged flash data until no more left.
// The retreival prints the time stamp and flash page number
// then outputs three, six or eight parameters per line.
// Data structure is as follows:
//    Data present in page - 'y' else anything else
//    Time stamp - day,hour,min,second,msec 
void RetreiveFlashData(void)
{
     unsigned char notfinished,msec,second,min,hour,day,loggeddata,combinedbyte;
 	 unsigned char i,j,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 unsigned char highbyte,lowbyte,inchar;
	 int xaxis,yaxis,zaxis,agyro,bgyro,cgyro,pagecnt,lograte,buflength;
	 int xmin,ymin,zmin,xmax,ymax,zmax;
	 int data[8];
	 long recordcnt;
	 float floattime,xgrav,ygrav,zgrav,xgmin,ygmin,zgmin,xgmax,ygmax,zgmax;
	 
	 Release_SPI();
	 Init_SPI_Master();
	 //  Set PC4 low - Flash CS
	 FlashChipSelect(LOW);  
	 SPI_StartReadFlashPage(0);
	 pagecnt = 1;
	 recordcnt = 1;
	 notfinished = YES;
	 xmin = ymin = zmin = 1024;
	 xmax = ymax = zmax = 0;
	 while((notfinished==YES)) {
	 	 inchar = getcharnw();
		 // Pause output until any key hit
		 if(inchar=='p') while((getcharnw()==0)) ;
		 // Halt output and exit data log dump
		 else if(inchar!=0) break;
	     byte = SPI_MasterReceive();
		 // See if first page byte is 'y' - process page if it is
		 if(byte=='y') {
		     combinedbyte = SPI_MasterReceive();
			 loggeddata = Convertdata2log(combinedbyte);
		     // Retrieve log rate
			 lograte = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
		     // Retrieve time stamp
			 day = SPI_MasterReceive();
			 hour = SPI_MasterReceive();
			 min = SPI_MasterReceive();
			 second = SPI_MasterReceive();
			 msec = SPI_MasterReceive();
			 
			 if((pagecnt==1)&&(logoutputmode==VERBOSE))  {
			     DisplaySensorID(combinedbyte);
				 DisplayAccelerometerRange(combinedbyte);
			     printf("%c %4d msec\n\r",loggeddata,lograte);
				 pagecnt++;
				 }
			 floattime = (float) second + ((float) (msec*4))/1000.0;
			 
			 if((loggeddata=='a')||(loggeddata=='g')) {		 
			     if(flashtype0==AT45DB081ID) buflength = 41;
	 		     else if(flashtype0==AT45DB642ID) buflength = 169;

			     for(i=0; i<buflength; i++) {
			         xaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         yaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         zaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();	
				     if(logoutputmode!=MINMAX) {
					     printf("%ld ",recordcnt++);
			             printf("%d:%d:%d:%#2.3f",day,hour,min,floattime);
						 }
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 if((logoutputmode==PROCESSED)||(logoutputmode==MINMAX)) {
					 	 if(xaxis>xmax)  xmax = xaxis;
						 if(yaxis>ymax)  ymax = yaxis;
						 if(zaxis>zmax)  zmax = zaxis;
						 if(xaxis<xmin)  xmin = xaxis;
						 if(yaxis<ymin)  ymin = yaxis;
						 if(zaxis<zmin)  zmin = zaxis;			

						 xgrav = ConvertXgrav(xaxis);
						 ygrav = ConvertYgrav(yaxis);
						 zgrav = ConvertZgrav(zaxis);
						 if(logoutputmode!=MINMAX) printf(" %# 6.3f %# 6.3f %# 6.3f\n\r",xgrav,ygrav,zgrav);
					     } 
					 else if(logoutputmode!=MINMAX) printf(" %4d %4d %4d\n\r",xaxis,yaxis,zaxis);
				     }
			     SPI_MasterReceive();			 		 		 
		         }
			 else if(loggeddata=='b') {
			     if(flashtype0==AT45DB081ID) buflength = 20;
	 		     else if(flashtype0==AT45DB642ID) buflength = 84;

			     for(i=0; i<buflength; i++) {
			         xaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         yaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         zaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					 agyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					 bgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					 cgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();	
				     printf("%ld ",recordcnt++);
			         printf("%d:%d:%d:%2.3f",day,hour,min,floattime);
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 printf(" %4d %4d %4d ",xaxis,yaxis,zaxis);
					 printf("%4d %4d %4d\n\r",agyro,bgyro,cgyro);
				     }
				 // Skip the blank bytes at the end of the page
				 for(i=0; i<7; i++ )  SPI_MasterReceive();			 		 		 
		         }

			 else if(loggeddata=='8') {
			     if(flashtype0==AT45DB081ID) buflength = 15;
	 		     else if(flashtype0==AT45DB642ID) buflength = 63;

			     for(i=0; i<buflength; i++) {
				     for(j=0; j<8; j++)  {
					     //lowbyte = SPI_MasterReceive();
						 //highbyte = SPI_MasterReceive();
						 //bgyro = highbyte*256 + lowbyte;
					     //if(i==0) printf("%x %x %d\n\r",highbyte,lowbyte,bgyro);
					     bgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					     data[j] = bgyro;
						 }					      
				     printf("%ld ",recordcnt++);
			         printf("%d:%d:%d:%2.3f",day,hour,min,floattime);
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 printf(" %4d %4d %4d ",data[0],data[1],data[2]);
					 printf(" %4d %4d %4d ",data[3],data[4],data[5]);
					 printf("%4d %4d\n\r",data[6],data[7]);
				     }
				 // Skip the blank bytes at the end of the page
				 for(i=0; i<7; i++ )  SPI_MasterReceive();			 		 		 
		         }
			  }
		 else {
		     if(logoutputmode==VERBOSE)  printf("Finished processing pages");
			 notfinished = NO;
			 xgrav = ConvertXgrav(xmax);
			 ygrav = ConvertYgrav(ymax);
			 zgrav = ConvertZgrav(zmax);
			 printf("\n\rMax G: %# 6.3f %# 6.3f %# 6.3f\n\r",xgrav,ygrav,zgrav);
			 xgrav = ConvertXgrav(xmin);
			 ygrav = ConvertYgrav(ymin);
			 zgrav = ConvertZgrav(zmin);
			 printf("Min G: %# 6.3f %# 6.3f %# 6.3f\n\r",xgrav,ygrav,zgrav);
			 } 
		 }
     
	 // Set PC4 high - deselect Flash chip
	 FlashChipSelect(HIGH); 
	 Release_SPI(); 
}

void FlashChipErase(void)
{
 	 Release_SPI();
	 Init_SPI_Master();
	 printf("\n\rErase serial flash chip");
	 ChipErase();
	 Release_SPI();
	 Init_SPI_Master();
	 WaitForFlashFree();
	 Release_SPI();
}

const char dislogmes1[] = "\n\rData logged is ";
const char dislogmes2[] = "accelerometer only\n\r";
const char dislogmes3[] = "rate gyro only\n\r";
const char dislogmes4[] = "accelerometer and rate gyro\n\r";
const char dislogmes5[] = "all 8 analog channels\n\r";
const char dislogmes6[] = "unknown selection\n\r";

void DisplayData2Log(void)
{
	 putfs(dislogmes1);
	 if(data2log=='a') putfs(dislogmes2);
	 else if(data2log=='g') putfs(dislogmes3);
	 else if(data2log=='b') putfs(dislogmes4);
	 else if(data2log=='8') putfs(dislogmes5);
	 else putfs(dislogmes6);
}	 

//const char loggermes1[] = "\n\rEntering log data to serial flash mode\n\r";
const char loggermes1[] = "MEMs sensor logging system\n\r";
const char loggermes2[] = "Default log rate is ";
const char loggermes3[] = "Default accelerometer range is ";
const char loggermes35[] = "\n\rDefault termination mode is ";
const char loggermes4[] = "Commands:\n\r";
const char loggermes5[] = "  a - Arm switch/disarm PD2 - PD2\n\r";
const char loggermes55[] = "  t - Terminate mode (b,t)\n\r";
const char loggermes6[] = "  m - Accelerometer range (1.5,2,4,6g)\n\r";
const char loggermes7[] = "  s - Start data logging\n\r";
const char loggermes8[] = "  h - Halt data logging\n\r";
const char loggermes85[] = "  i - Enter device id name\n\r";
const char loggermes9[] = "  e - Erase chip\n\r";
const char loggermes10[] = "  d - Select data to log (accelerometer,gyro,both,analog)\n\r";
const char loggermes11[] = "  r - Retrieve data\n\r";
const char loggermes12[] = "  l - Log rate (msec)\n\r";
const char loggermes13[] = "  f - Output format (raw,processed,minmax)\n\r";
const char loggermes135[] = "  c - Display command summary\n\r";
const char loggermes14[] = "  q - Quit data logging mode\n\r";
const char loggermes145[] = "\n\rEnter termination mode [b-button,t-time]: ";
const char loggermes15[] = "\n\rSelect accelerometer range [1-1.5g,2-2g,4-4g or 6-6g]: ";
const char loggermes16[] = "\n\rLogging mode started";
const char loggermes17[] = "\n\rSelect data to log [a-accelerometer, g-gyro, b-both, 8-all analog]: ";
const char loggermes18[] = "\n\rLogging halted";
const char loggermes19[] = "\n\rEnter new log rate (msec): ";
const char loggermes195[] = "\n\rEnter new device name [20 chars]: ";
const char loggermes21[] = "\n\rRetreive logged data\n\r";
const char loggermes22[] = "\n\rSelect output format [r-raw,p-processed,m-minmax]: ";

void DisplayCommandSummary()
{
	 putfs(loggermes4);
	 putfs(loggermes5);
	 putfs(loggermes55);
	 putfs(loggermes6);
	 putfs(loggermes7);
	 putfs(loggermes8);
	 putfs(loggermes85);
	 putfs(loggermes9);
	 putfs(loggermes10);
	 putfs(loggermes11);
	 putfs(loggermes12);
	 putfs(loggermes13);
	 putfs(loggermes135);
	 putfs(loggermes14);
}

// This function manages the real time logging of sample data
// to the serial flash chip, starts/stops the logging and retrieves
// logged data
void LogDataToSerialFlash()
{
 	 char inchar,inchar1,inchar2;
	 int i,delayperiod;
	 
// 	 putfs(loggermes1);
	 printf("\n\r");
	 DisplaySensorID2(); putfs(loggermes1);
 	 putfs(loggermes2);
	 printf("%d msec per sample\n\r",defaultlograte);
	 putfs(loggermes3);
	 if(defaultaccelerometerrange=='1') printf("1.5 g");
	 else printf("%c g",defaultaccelerometerrange);
	 putfs(loggermes35);
	 if(terminationmode==BUTTON) printf("button");
	 else printf("%d second timeout",timeoutperiod);
	 DisplayData2Log();
	 DisplayCommandSummary();
	 putchar('>');
	 
	 inchar = getcharr();
	 putchar(inchar);
	 inchar2 = 'g';
	 mseccountdown3 = delayperiod = 10000;  // exit after ten seconds of no activity 
	 while((inchar!='q')&&(inchar2!='b'))  {
	     if(mseccountdown3==0)  break;
	     switch(inchar) {
	         case 'a':
			     mseccountdown3 = delayperiod;
			     if(armpd2==YES) {
				     printf("\n\rDisarm PD2");
				     armpd2 = NO;
					 SaveEepromPreferences();
					 }
				 else {
				     printf("\n\rArm PD2");
				     armpd2 = YES;
					 SetBit(PORTD,PIND2);  // Activate pullup
					 SaveEepromPreferences();
					 }
		         break;
	         case 't':
			 	 mseccountdown3 = delayperiod;  
			     putfs(loggermes145);
				 terminationmode = getchar();
				 putchar(terminationmode);
				 if(terminationmode==TIMEOUT) {
				 	 printf("\n\rEnter timeout delay period (sec): ");
					 timeoutperiod = readint();
					 SaveEepromPreferences();
					 }
		         break;
	         case 'm':
			     mseccountdown3 = delayperiod;  
				 putfs(loggermes15);
				 defaultaccelerometerrange = getchar();
				 putchar(defaultaccelerometerrange);
				 SaveEepromPreferences();
		         break;
	         case 'e':
			     mseccountdown3 = delayperiod;  
			     GREEN_LED(ON);
			     FlashChipErase();
				 GREEN_LED(OFF);
		         break;
		     case 's':
			     mseccountdown3 = delayperiod;  
			 	 Init_3Axis_Accelerometer(defaultaccelerometerrange);
				 GREEN_LED(ON);
				 if(FlashBlankCheck()==YES) {
				     RED_LED(ON);
				     FlashChipErase();
					 RED_LED(OFF);
					 }
			     loggingmode = START;
				 if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
				 bufferidptr = 1;
				 pagecnt = 1;
				 inchar2 = 'b';
				 flashaddress = 0;
				 putfs(loggermes16);
		         break;
			 case 'd':
			     mseccountdown3 = delayperiod;  
				 putfs(loggermes17);
				 data2log = getchar();
				 putchar(data2log);
				 SaveEepromPreferences();
				 DisplayData2Log();
				 break;
		     case 'h':
			     mseccountdown3 = delayperiod;  
			     loggingmode = HALT;
				 GREEN_LED(OFF);
				 putfs(loggermes18);
		         break;
		     case 'i':
			     mseccountdown3 = delayperiod;  
			     putfs(loggermes195);
			     strcpy(deviceidtext,"                    ");
				 for(i=0; i<20; i++)  {
				     inchar = getchar();
					 putchar(inchar);
					 if(inchar==CR) break;
					 else deviceidtext[i] = inchar;
					 }
				 SaveEepromPreferences();
				 printf("\n\rDevice name changed to %s",deviceidtext);
		         break;
			 case 'l':
			     mseccountdown3 = delayperiod;  
			     putfs(loggermes19);
			     defaultlograte = readint();
				 SaveEepromPreferences();
				 printf("\n\rLog rate changed to %d msec",defaultlograte);
		         break;
		     case 'r':
			     mseccountdown3 = delayperiod;  
			     if(logoutputmode==VERBOSE)  putfs(loggermes21);
				 // Retreive logged data from each flash page
				 // until no more data left
				 RetreiveFlashData();
		         break;
	         case 'f':
			     mseccountdown3 = delayperiod;  
				 putfs(loggermes22);
				 logoutputmode = getchar();
				 putchar(logoutputmode);
				 SaveEepromPreferences();
		         break;
	         case 'c':
			     mseccountdown3 = delayperiod;  
			     putCRLF();
				 DisplayCommandSummary();
		         break;
		     }
		if(inchar2!='b') {
		    printf("\n\r>");
		    inchar = getcharr();
		    putchar(inchar);
			}
		}
}

// Combine device id into same byte as data logged
unsigned char CombinedeviceID(unsigned char data2log)
{
 	unsigned byte;
	
	if(data2log=='a') byte = 0;
	else if(data2log=='g') byte = 1;
	else if(data2log=='b') byte = 2;
	else if(data2log=='8') byte = 3;
	
	// Or with data2log value for recording
	if(deviceid==LEFTARM)  byte |= 0x10;  // left arm
	else if(deviceid==RIGHTARM)  byte |= 0x20;  // right arm
	else if(deviceid==UPPERSPINE)  byte |= 0x30;  // upper spine
	else if(deviceid==LOWERSPINE)  byte |= 0x40;  // lower spine
	else if(deviceid==SLED)  byte |= 0x50;  // sled

	// Or with accelerometer range setting
	if(defaultaccelerometerrange==G15MODE) byte |= 0x00;
	else if(defaultaccelerometerrange==G2MODE) byte |= 0x04;
	else if(defaultaccelerometerrange==G4MODE) byte |= 0x08;
	else if(defaultaccelerometerrange==G6MODE) byte |= 0x0c;
	
    return byte;
}

// Log data scheduler
// Always fill a flash page with data before returning
// This keeps the scheduler very simple
// This version logs the 3-axis accelerometer
void LogData(int logmsec)
{
	 int i,j,buflength;
	 union {
	    unsigned char tempbuf[2];
		int outiint;
		} u;
	 
	 Release_SPI();
	 Init_SPI_Master(); 	
	 
	 // Serial Flash CS high
	 FlashChipSelect(LOW);	
	  
//	 flashtype0 = SPI_SerialFlashStatus()&FLASHIDMASK;
	 
	
	 //  Select Buffer to write too
	 if(bufferidptr==1)  SPI_MasterTransmit(SF_WRITE_BUFFER1);
	 else  SPI_MasterTransmit(SF_WRITE_BUFFER2);
	 
	 //printf("ptr=%d ",bufferidptr);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(0);
	 //ConvertPower2FlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
 
	 SPI_MasterTransmit('y');
	 SPI_MasterTransmit(CombinedeviceID(data2log));
	 u.outiint = logmsec;
	 SPI_MasterTransmit(u.tempbuf[1]);
	 SPI_MasterTransmit(u.tempbuf[0]);
	 SPI_MasterTransmit(days);
	 SPI_MasterTransmit(hours); 
	 SPI_MasterTransmit(minutes);
	 SPI_MasterTransmit(seconds);
	 SPI_MasterTransmit(rtcclockcnt);
	 
//	 printf("Flashtype = %x\n\r",flashtype0);
	 
	 if((data2log=='a')||(data2log=='g'))  {
	     if(flashtype0==AT45DB081ID) buflength = 41;
	     else if(flashtype0==AT45DB642ID) buflength = 169;
	 
	     for(i=0; i<buflength; i++) {
		     if(data2log=='a') {
	             // X axis accelerometer
	             u.outiint = AnalogCh(5,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // Y axis accelerometer
	             u.outiint = AnalogCh(0,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // Z axis accelerometer
	             u.outiint = AnalogCh(1,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
	             }
			 else if(data2log=='g') {
	             // A axis gyro
	             u.outiint = AnalogCh(4,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // B axis gyro
	             u.outiint = AnalogCh(6,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // C axis gyro
	             u.outiint = AnalogCh(7,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
			     }
		    // Delay for the sample period
		    Delaymsec(logmsec);
			}
		 }
	 else if(data2log=='b') {
	     if(flashtype0==AT45DB081ID) buflength = 20;
	     else if(flashtype0==AT45DB642ID) buflength = 84;
	 
	     for(i=0; i<buflength; i++) {
	         // X axis accelerometer
	         u.outiint = AnalogCh(5,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // Y axis accelerometer
	         u.outiint = AnalogCh(0,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // Z axis accelerometer
	         u.outiint = AnalogCh(1,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
			 
	         // A axis gyro
	         u.outiint = AnalogCh(4,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // B axis gyro
	         u.outiint = AnalogCh(6,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // C axis gyro
	         u.outiint = AnalogCh(7,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		 
		     // Delay for the sample period
		     Delaymsec(logmsec);
	         }
		 }
	 else if(data2log=='8') {
	     if(flashtype0==AT45DB081ID) buflength = 15;
	     else if(flashtype0==AT45DB642ID) buflength = 63;
	 
	     for(i=0; i<buflength; i++) {
			 for(j=0; j<8; j++)  {
	             u.outiint = AnalogCh(j,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
				 }
			
		     // Delay for the sample period
		     Delaymsec(logmsec);
	         }	
		 }
		 
	 // Serial Flash CS high
	 FlashChipSelect(HIGH);
	 
	 // Store flash buffer to flashpage
	 FlashChipSelect(LOW);
	  
	 // Transfer buffer n to flash memory
	 if(bufferidptr==1) SPI_MasterTransmit(SF_BUF1_TO_FLASH_NOERASE);
	 else SPI_MasterTransmit(SF_BUF2_TO_FLASH_NOERASE);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(flashaddress);
	 //ConvertPower2FlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Progamming the flash can take up to 40msec so make sure the data 
	 // logging rate is not to high
	 FlashChipSelect(HIGH);
	 
	 // Flip the bufferid ptr
	 if(bufferidptr==1) bufferidptr = 2;
	 else bufferidptr = 1;

 	 if(flashtype0==AT45DB081ID)  {
	     flashaddress += 256;
		 if(++pagecnt>4096) loggingmode = HALT;
		 }
	 else if(flashtype0==AT45DB642ID)  {
	     flashaddress += 1024;
	 	 if(++pagecnt>8192)  loggingmode = HALT;
		 }

	 Release_SPI();
}

// Read all the analog inputs and output min/max status
void MeasureAnalogInputs(void)
{
    int i,j,min,max,analogvalue,analogtotal;
	
	printf("\n\rAnalog input summary\n\r");
	printf("Ch  average   min   max   diff\n\r");
	
	for(i=0; i<8; i++) {
		analogvalue = AnalogCh(i,EXTERNALREF);
		analogtotal = 0;
		min = max = analogvalue;
	    for(j=0; j<10; j++)  {
		    analogvalue = AnalogCh(i,EXTERNALREF);
			if(analogvalue < min)  min = analogvalue;
			if(analogvalue > max)  max = analogvalue;
			analogtotal += analogvalue;
			}
		printf("%4d     %4d    %4d    %4d    %4d",i,analogtotal/10,min,max,max-min);
		if(i==0) printf("  Y axis acceleration");
		else if(i==1) printf("  Z axis acceleration");
		else if(i==4) printf("  A Gyro");
		else if(i==5) printf("  X axis acceleration");
		else if(i==6) printf("  B Gyro");
		else if(i==7) printf("  C Gyro");
		printf("\n\r");
		}
}

float Read_BMP085_pressurefloat(char mode)
{	 
    unsigned int BMPpressure,BMPtemperature;
	long x1,x2,b5,temperatureinC;
	long b6,x3,b3,p;
	unsigned long b4,b7;
	float compensatedpressure;
	
	BMPpressure = BMPtemperature = 0x00;
/*	bmpac1 = 408; bmpac2 = -72; bmpac3 = -14383; bmpac4 = 32741;
	bmpac5 = 32757; bmpac6 = 23151; bmpb1 = 6190; bmpb2 = 4;
	bmpmb = -32768; bmpmc = -8711; bmpmd = 2868;
*/
	
	// Get raw temperature and compensate
 	BMPtemperature = Read_BMP085_Temperature();
	x1 = ((long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	temperatureinC = (b5 + 8)/16;
	
	// Get raw pressure and compensate
	BMPpressure = Read_BMP085_Pressure();
//	BMPpressure = 23843;
	
	b6 = b5 - 4000;
	//printf("b6 = %d\n\r",b6);
	x1 = ((long)bmpb2*((b6*b6)/4096))/2048;
	//printf("x1 = %d\n\r",x1);
	x2 = ((long)bmpac2*b6)/2048;
	//printf("x2 = %d\n\r",x2);
	x3 = x1 + x2;
	//printf("x3 = %d\n\r",x3);
	b3 = ((((long)bmpac1*4+x3)<<0)+2)/4;
	//printf("b3 = %d\n\r",b3);
	x1 = ((long)bmpac3*b6)/8192;
	//printf("x1 = %d\n\r",x1);
	x2 = ((long)bmpb1*((b6*b6)/4096))/65536;
	//printf("x2 = %d\n\r",x2);
	x3 = ((x1+x2)+2)/4;
	//printf("x3 = %d\n\r",x3);
	b4 = ((long)bmpac4*(unsigned long)(x3+32768))/32768;
	//printf("b4 = %u\n\r",b4);
	b7 = ((unsigned long)BMPpressure-b3)*((unsigned long)50000>>0);
	//printf("b7 = %lu\n\r",b7);
	if(b7<0x80000000) p = (b7*2)/b4;
	else p = (b7/b4)*2;
	//printf("p = %ld\n\r",p);
	x1 = (p/256)*(p/256);
	//printf("x1 = %ld\n\r",x1);
	x1 = (x1*3038)/65536;
	//printf("x1 = %d\n\r",x1);
	x2 = (-7357*p)/65536;
	//printf("x2 = %d\n\r",x2);
	p = p + (x1+x2+3791)/16;
	
// 	if(mode==VERBOSE) printf("\rBMP085 temperature raw = %u    \n\r",BMPtemperature);
// 	printf("\rBMP085 temperature (C)= %.2f    \n\r",(float)temperatureinC/10.0);
// 	if(mode==VERBOSE) printf("\rBMP085 pressure raw = %u    \n\r",BMPpressure);
// 	if(mode==VERBOSE) printf("\rBMP085 pressure (p) = %ld    \n\r",p);	
// 	printf("\rBMP085 pressure (psi) = %.4f    \n\r",(float)p*0.00014504);	
	compensatedpressure = (float)p*0.00014504;
	return compensatedpressure;
}

void DisplayBMP085(char mode)
{	 
    unsigned int BMPtemperature,i,reps;
	long x1,x2,b5,temperatureinC;
	float pressure;

	// Get raw temperature and compensate
 	BMPtemperature = Read_BMP085_Temperature();
	x1 = ((long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	temperatureinC = (b5 + 8)/16;
	
	pressure = 0;
	reps = 10;
	for(i=0; i<reps; i++) {
	    pressure += Read_BMP085_pressurefloat(QUIET);
		Delaymsec(100);
		}
	pressure = pressure/reps;
 	printf("\rBMP085 temperature (C)= %.2f    \n\r",(float)temperatureinC/10.0);
 	printf("\rBMP085 pressure (psi) = %.4f  %.1fft  \n\r",pressure, (14.696-pressure)/0.000407);	
}

void DisplayADS1100(void)
{	 unsigned int ADSreading;
	 ADSreading = 0x00;
	 Enumerate_ADS();
	 
	 Delaymsec(300);
	 printf("\n\r");
	 while(getcharnw()==0)
	 {
	     ADSreading = Read_ADS1100();
 	     printf("\rADS reading = %i    ",ADSreading); 
		 printf("%f    \n\r",((float)ADSreading/32767.0)*0.63*360.0);
	     Delaymsec(500);
	 }
}

char DisplayCompass(void)
{
 	 unsigned int digitalcompassheading = 180;
	 
	 digitalcompassheading = Read_DigitalCompass_Heading();
 	 printf("\n\rDigital compass heading = %.1f\n\r",(float)digitalcompassheading/10.0);	
}

void Delayseconds(int seconds)
{
 	 int i;
	 
 	 for(i=0; i<seconds; i++) Delaymsec(1000);
}

unsigned char mm130motorcontrollerstate=0;

void EnableBMP085(char select)
{
 	 if(select==BMPSENSOR1) {
	     SetBit(PORTD,PIND6);
		 ClrBit(PORTD,PIND7);
		 }
	 else if(select==BMPSENSOR2) {
		 ClrBit(PORTD,PIND6);
	     SetBit(PORTD,PIND7);
		 }
	 Delaymsec(30);
}

void DisableBMPs()
{
 	 ClrBit(PORTD,PIND6);
	 ClrBit(PORTD,PIND7);
}

void ClearSmartLCD()
{
 	 Init_SmartLCD_TWI(SMARTLCDTWIADDRESS);
	 Delaymsec(10);
}

void _StackOverflowed(char dunno)
{
 printf("\n\n\rStack overflow! Return: %i",dunno);
}

void DisplayCaliper(void)
{	  
    long int CaliperReading;
	float correctedValue;
	 CaliperReading = 0x00;

	// Delaymsec(300);
	 printf("\n");
	 while(1)
	{
	 CaliperReading = Read_caliper();
	 
	 Delaymsec(30);
 	//printf("Caliper reading = %i inch  \n\r",CaliperReading);
	//correctedValue = (int)CaliperReading;
	
	if (CaliperReading >= 0x100000)		   //if negative
	{
	   CaliperReading ^= 0xFFFFFF;		   //get the 1's inverse
	   CaliperReading*=-1;   
	}
	
	correctedValue = (float)CaliperReading/ 20480.0;
	
	 printf("\rActual Caliper reading = %2.3f inch   ",correctedValue);
	 
//	 _StackCheck();
	}
}
// Write float value to LED8 via TWI
void Write_Drive_MM130(char *outstrg)
{ 
	 TWI_Start_Write(MM130TWIADDRESS,outstrg,strlen(outstrg));
}

void DriveMM130()
{
 	 char inchar,outstrg[20];
	     
	 printf("\n\rInput command to send to MM165\n\r");
	 printf("Enter [1|2|3|4|5|6|7|8|l|h|e]: ");
	 inchar = '1';
 	 while((inchar=getchar())!='e') {
	     putchar(inchar);  putCRLF();
		 if(inchar=='1') Write_Drive_MM130("dd10.15;3.45;");
		 else if(inchar=='2') Write_Drive_MM130("dd10.00;3.45;");
		 else if(inchar=='3') Write_Drive_MM130("dd20.15;3.45;");
		 else if(inchar=='4') Write_Drive_MM130("dd20.00;3.45;");
		 else if(inchar=='5') Write_Drive_MM130("dd80.15;3.45;");
		 else if(inchar=='6') Write_Drive_MM130("dd80.00;3.45;");
		 else if(inchar=='7') Write_Drive_MM130("dd60.15;3.45;");
		 else if(inchar=='8') Write_Drive_MM130("dd60.00;3.45;");
		 else if(inchar=='l') Write_Drive_MM130("q2");
		 else if(inchar=='h') Write_Drive_MM130("q1");
	 	 printf("Enter [1|2|3|4|5|6|7|8|l|h|e]: ");
	     }
	 putCRLF();
}

// ADC2 inputs the battery voltage divider network
void BatteryVoltage()
{
     int batteryvoltage;
	 
 	 batteryvoltage = AnalogCh(2,INTERNALREF);
 	 batteryvoltage = AnalogCh(2,INTERNALREF);
 	 batteryvoltage = AnalogCh(2,INTERNALREF);
	 printf("\n\rBattery voltage = %d %1.3f volts\n\r",batteryvoltage,(float)batteryvoltage*0.00741);
// 	 batteryvoltage = AnalogCh(2,EXTERNALREF);
//	 printf("\n\rBattery voltage = %d %1.3f volts\n\r",batteryvoltage,(float)batteryvoltage*0.00375);
}

// Set ADXL345 chip select state - PB1
void ADXL345ChipSelect(unsigned char mode)
{
     shortdelay(2);
	 if(mode==LOW) PORTB &= 0xFD; 
	 else PORTB |= 0x02;  
     shortdelay(2);
}

//  Set ADXL345 data format mode
void SPI_ADXL345_DataFormat(void)
{
 	 unsigned char deviceid;
	 
	 //  Set PB1 - Flash CS
	 ADXL345ChipSelect(LOW);

	 // Send read status command
 	 SPI_MasterTransmit(DATAFORMATADDRESS);
 	 SPI_MasterTransmit(0x00);
	 
	 // Set PB1 high - deselect Flash chip
	 ADXL345ChipSelect(HIGH);
}

//  Write command to ADXL345
void SPI_ADXL345_Write(unsigned char commandadd,unsigned char data)
{
 	 unsigned char deviceid;
	 
	 //  Set PB1 - Flash CS
	 ADXL345ChipSelect(LOW);

	 // Send read status command
 	 SPI_MasterTransmit(commandadd);
 	 SPI_MasterTransmit(data);
	 
	 // Set PB1 high - deselect Flash chip
	 ADXL345ChipSelect(HIGH);
}

void ADXL345_SetRange(char inchar)
{
 	 if(inchar=='2') SPI_ADXL345_Write(DATAFORMATADDRESS,0x08); 
	 else if(inchar=='4') SPI_ADXL345_Write(DATAFORMATADDRESS,0x09);
	 else if(inchar=='8') SPI_ADXL345_Write(DATAFORMATADDRESS,0x0A);
	 else if(inchar=='6') SPI_ADXL345_Write(DATAFORMATADDRESS,0x0B);
}

// Read acceleration on an axis
int SPI_ADXL345_ReadAxis(unsigned char axisselect)
{
	 unsigned char lowbyte,highbyte;
	 int axisvalue,axisvalue2;
	 
	 //  Set PB1 - Flash CS
	 ADXL345ChipSelect(LOW);

	 if(axisselect==ADXL345XAXIS) {
 	     SPI_MasterTransmit((XAXISDATA0ADDRESS|0xC0));
	     lowbyte = SPI_MasterReceive();
	     highbyte = SPI_MasterReceive();
		 }
	 else if(axisselect==ADXL345YAXIS) {
 	     SPI_MasterTransmit((YAXISDATA0ADDRESS|0xC0));
	     lowbyte = SPI_MasterReceive();
	     highbyte = SPI_MasterReceive();
		 }
	 else if(axisselect==ADXL345ZAXIS) {
 	     SPI_MasterTransmit((ZAXISDATA0ADDRESS|0xC0));
	     lowbyte = SPI_MasterReceive();
	     highbyte = SPI_MasterReceive();
		 }

	 //printf("low=%x high=%x\n\r",lowbyte,highbyte);
	 axisvalue = 0;
	 axisvalue = lowbyte;
	 axisvalue2 = 0;
	 axisvalue2 = highbyte;
	 axisvalue2 = axisvalue2<<8;
	 axisvalue += axisvalue2;
	 
	 // Set PB1 high - deselect Flash chip
	 ADXL345ChipSelect(HIGH);
	 
	 return axisvalue;
}

//  Get ADXL345 device id
unsigned char SPI_ADXL345_ID(void)
{
 	 unsigned char deviceid;
	 
	 //  Set PB1 - Flash CS
	 ADXL345ChipSelect(LOW);

	 // Send read status command
 	 SPI_MasterTransmit((DEVICEIDADDRESS|0x80));

	 deviceid = SPI_MasterReceive();
	 //printf("device id 1 = %x\n\r",deviceid);
	 
	 // Set PB1 high - deselect Flash chip
	 ADXL345ChipSelect(HIGH);
	 
	 return deviceid;
}

// Display ADXL345 data continuously
void DisplayADXL345Data()
{
    int xaxisint,yaxisint,zaxisint,delay;
	float xgrav,ygrav,zgrav,xmin,xmax,ymin,ymax,zmin,zmax;
	
//	Init_SPI_Master();
	
	//printf("Display gravity - [s-update speed(ms),g-range,e-exit]\n\r");
	//printf("Range is %c\n\r",gmode);
	xmin = ymin = zmin = 16.2;
	xmax = ymax = zmax = -16.2;	
	delay = 1;
		xaxisint = SPI_ADXL345_ReadAxis(ADXL345XAXIS);
		yaxisint = SPI_ADXL345_ReadAxis(ADXL345YAXIS);
		zaxisint = SPI_ADXL345_ReadAxis(ADXL345ZAXIS);
		
		xgrav = (float)xaxisint * 0.0039;	
		ygrav = (float)yaxisint * 0.0039;
		zgrav = (float)zaxisint * 0.0039;
		
		if(xmin>xgrav) xmin = xgrav;
		if(ymin>ygrav) ymin = ygrav;
		if(zmin>zgrav) zmin = zgrav;
		if(xmax<xgrav) xmax = xgrav;
		if(ymax<ygrav) ymax = ygrav;
		if(zmax<zgrav) zmax = zgrav;
		
		//printf("xaxis = %4d ",xaxisint);
		printf("%# 1.3f ",xgrav);
		//printf("yaxis = %4d ",yaxisint);
		printf("%# 1.3f ",ygrav);
		//printf("zaxis = %4d ",zaxisint);
		printf("%# 1.3f  ",zgrav);
		if(0) {
		    printf("%# 1.3f ",xmin);
		    printf("%# 1.3f ",ymin);
		    printf("%# 1.3f ",zmin);
		    printf("%# 1.3f ",xmax);
		    printf("%# 1.3f ",ymax);
		    printf("%# 1.3f ",zmax);	
			}
}

// Set L3G4200D chip select state - PB2
void L3G4200DChipSelect(unsigned char mode)
{
     shortdelay(2);
	 if(mode==LOW) PORTB &= 0xFB; 
	 else PORTB |= 0x04;  
     shortdelay(2);
}

//  Get L3G4200D register value
void SPI_L3G4200D_PutByte(unsigned char iregister,unsigned char outbyte)
{
 	 unsigned char regvalue;
	 
	 //  Set PB1 - Flash CS
	 L3G4200DChipSelect(LOW);

 	 SPI_MasterTransmit(iregister);
 	 SPI_MasterTransmit(outbyte);
	 
	 // Set PB1 high - deselect Flash chip
	 L3G4200DChipSelect(HIGH);
}

//  Get L3G4200D register value
unsigned char SPI_L3G4200D_GetByte(unsigned char iregister)
{
 	 unsigned char regvalue;
	 
	 //  Set PB1 - Flash CS
	 L3G4200DChipSelect(LOW);

	 // Send who am i command - return value should be D2
 	 SPI_MasterTransmit((iregister|0x80));

	 regvalue = SPI_MasterReceive();
	 //printf("device id = %x\n\r",deviceid);
	 
	 // Set PB1 high - deselect Flash chip
	 L3G4200DChipSelect(HIGH);
	 
	 return regvalue;
}

//  Get ADXL345 device id
unsigned char SPI_L3G4200D_ID(void)
{
 	 unsigned char deviceid;
	 
	 //  Set PB1 - Flash CS
	 L3G4200DChipSelect(LOW);

	 // Send who am i command - return value should be D2
 	 SPI_MasterTransmit((WHO_AM_I|0x80));

	 deviceid = SPI_MasterReceive();
	 //printf("device id = %x\n\r",deviceid);
	 
	 // Set PB1 high - deselect Flash chip
	 L3G4200DChipSelect(HIGH);
	 
	 return deviceid;
}

void DumpHMC5883Regs()
{
    unsigned char i;
	
    printf("\n\rHMC5883 registers:\n\r");
	for(i=0; i<13; i++) printf(" reg %d: %x\n\r",i,Read_HMC5883_Register(i));
}

const char message9[] = "\n\rCheck DOF10W presence\n\r";
const char message10[] = " ADXL345 found\n\r";
const char message11[] = " ADXL345 not found\n\r";
const char message12[] = " L3G4200D found\n\r";
const char message13[] = " L3G4200D not found\n\r";
const char message14[] = " HMC5883 found\n\r";
const char message15[] = " HMC5883 not found\n\r";
const char message16[] = " BMP085 found\n\r";
const char message17[] = " BMP085 not found\n\r";

// Check for 10 Degree of freedom DOF10W
void CheckforDOF10W()
{
 	unsigned char ibyte;
	
    putfs(message9);
	Init_SPI_Master();

    // Check for ADXL345
	if(SPI_ADXL345_ID()==0xE5) {
	    // Init ADXL345
		SPI_ADXL345_Write(DATAFORMATADDRESS,0x0B);
		SPI_ADXL345_Write(POWERCONTROLADDRESS,0x08);
		SPI_ADXL345_Write(INTENABLEADDRESS,0x80);
	    putfs(message10);
		}
	else putfs(message11);
	
	// Check for L3G4200
	if(SPI_L3G4200D_ID()==0xD3) {
	    putfs(message12);
		SPI_L3G4200D_PutByte(CTRL_REG1,0x0f);
		}
	else putfs(message13);
	
	// Check for HMC5883
	Enumerate_HMC5883();
	if(hmc5883state!=0) {
	    putfs(message14);
		ibyte = Read_HMC5883_Register(HMC5883_CONFIG_REGA);
		Write_HMC5883_Register(HMC5883_CONFIG_REGA,(ibyte&0x7F));
		//DumpHMC5883Regs();
		}
	else putfs(message15);
	
	// Check for BMP085
	Enumerate_BMP085();
	if(BMPstate!=0) {
		Load_BMP085_Parameters();		
	    putfs(message16);
		}
	else putfs(message17);
}

void DisplayMagnetometer()
{
    unsigned char i;
	int magx,magy,magz;
	float magxf,magyf,magzf,magvector;
	union {
	    unsigned char tempbuf[2];
		int outint;
		} mag;

	
	for(i=3; i<=8; i++) printf("%x ",Read_HMC5883_Register(i));
	magx = magy = magz = 0;
	mag.tempbuf[0] = Read_HMC5883_Register(HMC5883_OUT_X_L);
	mag.tempbuf[1] = Read_HMC5883_Register(HMC5883_OUT_X_H);
	magx = mag.outint;
	mag.tempbuf[0] = Read_HMC5883_Register(HMC5883_OUT_Y_L);
	mag.tempbuf[1] = Read_HMC5883_Register(HMC5883_OUT_Y_H);
	magy = mag.outint;
	mag.tempbuf[0] = Read_HMC5883_Register(HMC5883_OUT_Z_L);
	mag.tempbuf[1] = Read_HMC5883_Register(HMC5883_OUT_Z_H);
	magz = mag.outint;
	printf("%d ",magx);
	printf("%d ",magy);
	printf("%d ",magz);
	magxf = ((float) magx)*1.3/2048.0;
	magyf = ((float) magy)*1.3/2048.0;
	magzf = ((float) magz)*1.3/2048.0;
	magvector = sqrt((magxf*magxf)+(magyf*magyf)+(magzf*magzf));
	printf("%#6.3f ",magxf);
	printf("%#6.3f ",magyf);
	printf("%#6.3f ",magzf);
	printf("%#6.3f ",magvector);
}

void DisplayGyroL3G4200()
{
 	unsigned char i;
	int gyrox,gyroy,gyroz;
	float gyroxf,gyroyf,gyrozf;
	union {
	    unsigned char tempbuf[2];
		int outint;
		} gyro;
		
	for(i=OUT_X_L; i<=OUT_Z_H; i++) printf("%x ",SPI_L3G4200D_GetByte(i));
	gyrox = gyroy = gyroz = 0;
	gyro.tempbuf[0] = SPI_L3G4200D_GetByte(OUT_X_L);
	gyro.tempbuf[1] = SPI_L3G4200D_GetByte(OUT_X_H);
	gyrox = gyro.outint;
	gyro.tempbuf[0] = SPI_L3G4200D_GetByte(OUT_Y_L);
	gyro.tempbuf[1] = SPI_L3G4200D_GetByte(OUT_Y_H);
	gyroy = gyro.outint;
	gyro.tempbuf[0] = SPI_L3G4200D_GetByte(OUT_Z_L);
	gyro.tempbuf[1] = SPI_L3G4200D_GetByte(OUT_Z_H);
	gyroz = gyro.outint;
	printf("%d ",gyrox);
	printf("%d ",gyroy);
	printf("%d ",gyroz);
	gyroxf = ((float) gyrox)*250.0/32768.0;
	gyroyf = ((float) gyroy)*250.0/32768.0;
	gyrozf = ((float) gyroz)*250.0/32768.0;
	printf("%#6.3f ",gyroxf);
	printf("%#6.3f ",gyroyf);
	printf("%#6.3f ",gyrozf);
}

void DumpL3G4200Registers()
{
 	 unsigned char i;
	 printf("\n\rL3G4200D Registers\n\r");
	 printf(" reg 0x0f: %x\n\r",SPI_L3G4200D_GetByte(0x0f));
	 for(i=0x20; i<=0x38; i++)  printf(" reg %x: %x\n\r",i,SPI_L3G4200D_GetByte(i));
}

void DisplayBMP085Brief()
{
    unsigned int BMPtemperature;
	long x1,x2,b5,temperatureinC;
	
    // Get raw temperature and compensate
 	BMPtemperature = Read_BMP085_Temperature();
	x1 = ((long)(BMPtemperature - (long)bmpac6) * (long)bmpac5)/(long)32768;
	x2 = ((long)bmpmc*2048)/(x1+(long)bmpmd);
	b5 = x1 + x2;
	temperatureinC = (b5 + 8)/16;
	
 	printf("%.2f ",(float)temperatureinC/10.0);
 	printf("%.4f ",Read_BMP085_pressurefloat(QUIET));	
}

// Display accelerometer, gyro, magnetometer and pressure
void DisplayDOF10WData()
{	
	char inchar;
    unsigned int BMPtemperature;
	int gyrox,gyroy,gyroz,xaxisint,yaxisint,zaxisint,delay;
	int magx,magy,magz;
	long x1,x2,b5,temperatureinC;
	float gyroxf,gyroyf,gyrozf,xgrav,ygrav,zgrav;
	float magxf,magyf,magzf;
	union {
	    unsigned char tempbuf[2];
		int outint;
		} gyro;
	union {
	    unsigned char tempbuf[2];
		int outint;
		} mag;

	delay = 1;
	//printf("xa  ya   za    xg    yg   zg   xm   ym   zm   t  psi\n\r");

		// Display accelerometer data
		xaxisint = SPI_ADXL345_ReadAxis(ADXL345XAXIS);
		yaxisint = SPI_ADXL345_ReadAxis(ADXL345YAXIS);
		zaxisint = SPI_ADXL345_ReadAxis(ADXL345ZAXIS);
		
		xgrav = (float)xaxisint * 0.0039;	
		ygrav = (float)yaxisint * 0.0039;
		zgrav = (float)zaxisint * 0.0039;

		printf("%# 1.3f ",xgrav);
		printf("%# 1.3f ",ygrav);
		printf("%# 1.3f  ",zgrav);

	    // Display L3G4200D gyro data
		gyrox = gyroy = gyroz = 0;
		gyro.tempbuf[0] = SPI_L3G4200D_GetByte(OUT_X_L);
		gyro.tempbuf[1] = SPI_L3G4200D_GetByte(OUT_X_H);
		gyrox = gyro.outint;
		gyro.tempbuf[0] = SPI_L3G4200D_GetByte(OUT_Y_L);
		gyro.tempbuf[1] = SPI_L3G4200D_GetByte(OUT_Y_H);
		gyroy = gyro.outint;
		gyro.tempbuf[0] = SPI_L3G4200D_GetByte(OUT_Z_L);
		gyro.tempbuf[1] = SPI_L3G4200D_GetByte(OUT_Z_H);
		gyroz = gyro.outint;
		gyroxf = ((float) gyrox)*250.0/32768.0;
		gyroyf = ((float) gyroy)*250.0/32768.0;
		gyrozf = ((float) gyroz)*250.0/32768.0;
		printf("%#6.1f ",gyroxf);
		printf("%#6.1f ",gyroyf);
		printf("%#6.1f ",gyrozf);
	
	    // Display magnetometer data
		mag.tempbuf[0] = Read_HMC5883_Register(HMC5883_OUT_X_L);
		mag.tempbuf[1] = Read_HMC5883_Register(HMC5883_OUT_X_H);
		magx = mag.outint;
		mag.tempbuf[0] = Read_HMC5883_Register(HMC5883_OUT_Y_L);
		mag.tempbuf[1] = Read_HMC5883_Register(HMC5883_OUT_Y_H);
		magy = mag.outint;
		mag.tempbuf[0] = Read_HMC5883_Register(HMC5883_OUT_Z_L);
		mag.tempbuf[1] = Read_HMC5883_Register(HMC5883_OUT_Z_H);
		magz = mag.outint;
		//printf("%d ",magx);
		//printf("%d ",magy);
		//printf("%d ",magz);
		magxf = ((float) magx)*1.3/2048.0;
		magyf = ((float) magy)*1.3/2048.0;
		magzf = ((float) magz)*1.3/2048.0;
		printf("%#6.3f ",magxf);
		printf("%#6.3f ",magyf);
		printf("%#6.3f ",magzf);
		
		// Display barometer data
		DisplayBMP085Brief();
		
		printf(" \r");
}

void DisplayDOF10WValues(unsigned char inchar)
{
    char inchar2;
	int delay=40;
	
    while((inchar2=getcharnw())!='e') {
		if(inchar2=='s') {
		    printf("Enter speed (msec): ");
			delay = readint();
			}
	    if(inchar=='a')  {
		    DisplayADXL345Data();
			putchar('\r');
			}
		if(inchar=='g')  {
		    DisplayGyroL3G4200();
			putchar('\r');
			}
		if(inchar=='m')  {
		    DisplayMagnetometer();
			putchar('\r');
			}
		if(inchar=='b')  {
		    DisplayBMP085Brief();
			putchar('\r');
			}
		if(inchar=='1')  {
		    printf("\n\r");
		    DisplayDOF10WData();
			putchar('\r');
			}
	    Delaymsec(delay);
		}
}

void DisplayDOF10WRegisters(unsigned char inchar)
{
    char inchar2;
	
	if(inchar=='a') ;
    if(inchar=='g')  DumpL3G4200Registers();
	if(inchar=='m')  DumpHMC5883Regs();
	if(inchar=='b')  ;
}

const char dof10mess1[] = "\n\rDOF10 Menu\n\r";
const char dof10mess2[] = "da - Display Acceleration\n\r";
const char dof10mess3[] = "dg - Display Gyro\n\r";
const char dof10mess4[] = "dm - Display Magnetometer\n\r";
const char dof10mess5[] = "db - Display Barometer\n\r";
const char dof10mess6[] = "d1 - Display All\n\r";
const char dof10mess7[] = "ra - Dump Acceleration Registers\n\r";
const char dof10mess8[] = "rg - Display Gyro Registers\n\r";
const char dof10mess9[] = "rm - Display Magnetometer Registers\n\r";
const char dof10mess10[] = "rb - Display Barometer Registers\n\r";

void DOF10WMenu()
{
 	char inchar1,inchar2;
	
    putfs(dof10mess1);
    putfs(dof10mess2);
    putfs(dof10mess3);
    putfs(dof10mess4);
    putfs(dof10mess5);
    putfs(dof10mess6);
    putfs(dof10mess7);
    putfs(dof10mess8);
    putfs(dof10mess9);
	printf("\n\rDOF-");
	while((inchar1=getchar())!='e') {
	    putchar(inchar1);
	    inchar2 = getchar();
		putchar(inchar2);
		printf("\n\r");
		if(inchar1=='d') DisplayDOF10WValues(inchar2);
		if(inchar1=='r') DisplayDOF10WRegisters(inchar2);
	    printf("\n\rDOF-");
	    }
}

//*************************************************************
//  Main work starts here
//*************************************************************
void main()
{
	unsigned char inchar,inbyte,twistatus1,twistatus2,twistatus3,twistatus4;
	unsigned char inbytes[5],toggle,slaverx,firstmeasurement,numaxis,lowbyte,highbyte;
	int i,icnt,j,loopsetting,loopmode,blackfinresetmode,countervalue,numbr;
	int loopmode2,num1,num2,xgrav,ygrav,xmin,xmax,iforce,dataoutputperiod;
	int ylocation,centerpressed,delaycount,xaxisint,yaxisint,zaxisint;
	int axisvalue,axisvalue2;
	long xgravity,ygravity,loopcount,ivalue,bvalue,cvalue;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
			
RESTART:
	Init_AVR();
	rxcntmax = totalrxcnt = 0;
	
	loopsetting = 500;   // 1/2 second delay
	loopmode = 0;  toggle = 0;
	mseccountdown1 = loopsetting;
	loopcount = 0;
	dataoutputperiod = 10;  // seconds
	//LCD_Switches = 0x00;   // Switch toggle values
	_textmode = 0;
	
	// Insert copyright - version reference in Flash memory
	strcpy(version,"SOC Wasp  R0.94 (c) 2010");

	// Get Serial Flash status
//	CheckFlashTypes();
	
	// Show something is happening
	while(0) {  
		BlinkRed(5);
		BlinkGreen(5);
		}
		
	_SEI(); // Enable interrupts => enable UART interrupts

	BlinkRed(2);
	BlinkGreen(2);
	
	Init_3Axis_Accelerometer(G15MODE);

	//Nokia_LCD_LED(ON);
	//Delaymsec(500);
	//Nokia_LCD_LED(OFF);
	
	signon();
	
	//Nokia_LCD_LED(ON);
	//Initialize_Nokia_LCD();			
	
	//Draw_Menu_Command();
	
	if(i2cmode==ACTIVE) Enumerate_TWI_Devices();
	if(dof10wmode==ACTIVE) CheckforDOF10W();
	
	printf("\n\r-");
	
	// Wait for commands on serial channel
	while (1) {
	
		// The log data function is started
		if(loggingmode==START) {
		    LogData(defaultlograte);
			if((terminationmode==TIMEOUT)&&(rtccountdown4==0)) {
			    GREEN_LED(ON);
				Delaymsec(3000);
				loggingmode = HALT;
			    printf("\n\rStop logging sensor data\n\r-");
			    GREEN_LED(OFF);
				}
			}
		
		// Turn logging off if button pressed again
		if((armpd2==YES)&&(!(PIND&(1<<PIND2)))&&(loggingmode==START)) {
		    delaycount = 0;
			GREEN_LED(ON);
		    while((!(PIND&(1<<PIND2)))&&(delaycount<4000)) {
				delaycount += 10;
			    Delaymsec(10);
				}
				
			//printf("turn off ");
			// If push button high haven't waited long enough
			if(delaycount>=4000) {
			    //printf("got past break ");
		        loggingmode = HALT;
			    printf("\n\rStop logging sensor data\n\r-");
			    GREEN_LED(OFF);
			    // wait until switch is released
			    while(!(PIND&(1<<PIND2))) ;
			    Delaymsec(200);
			    }
			}
		
		// If logging hasn't started see if button pressed
		if((armpd2==YES)&&(!(PIND&(1<<PIND2)))) {
		    loggingmode = START;
			elapsedtime = 0;
			printf("\n\rStart logging sensor data");
			Init_3Axis_Accelerometer(defaultaccelerometerrange);
			GREEN_LED(ON);
			if(FlashBlankCheck()==YES) {
				RED_LED(ON);
				FlashChipErase();
				RED_LED(OFF);
				}
			if(terminationmode==TIMEOUT) rtccountdown4 = timeoutperiod;
			printf("\n\r-");
			bufferidptr = 1;
			pagecnt = 1;
			flashaddress = 0;
			// Halt until switch is released
			while(!(PIND&(1<<PIND2))) ;
			Delaymsec(200);
			}

		// Work starts here
		if((inchar = getcharnw())) {
				
		        if(0) ProcessLCDCommands(inchar);
				
			   	switch (inchar) {
					case 'q': case 'Q':
					     putchar(inchar);
						 DisplayCaliper();
					     putcharr('-');
						 break;
					case 'f':
					     putchar(inchar);
						 DriveMM130();
						 putcharr('-');
						 break;
					case '&':
					     putchar(inchar);
						 goto RESTART;
						 break;
					case 'b': case 'B':
					     putchar(inchar);
						 BatteryVoltage();
					     putcharr('-');
						 break;
					case 'z': case 'Z':
					     putchar(inchar);
						 DisplayADS1100();
					     printf("\n\r-");
						 break;
					case 's': case 'S':
					     putchar(inchar);
						 signon();
						 if(i2cmode==ACTIVE)  Enumerate_TWI_Devices();
						 if(dof10wmode==ACTIVE) CheckforDOF10W();
						 printf("\n\r-");
						 //printf("rxcnt=%d\n-",rxcnt); 
						 break;
					case 'm': case 'M':
						  putcharr(inchar);
						  MeasureAnalogInputs();
					      putcharr('-');
						  break;
					case 'd': case 'D':
						  putcharr(inchar);
						  DisplayMemory();
					      putcharr('-');
						  break;
					case 'c': case 'C':
						  putcharr(inchar);
						  DisplayCompass();
					      putcharr('-');
						  break;
					case 'p': case 'P':
							putcharr(inchar);
						    LoadMemory();
							putcharr('-');
							break;
					case 'a': case 'A':
						    DumpAnalogChs();
							xgravity = ygravity = 0;
							for(i=0; i<10; i++) {
								xgravity += AnalogCh(0,EXTERNALREF);
								ygravity += AnalogCh(1,EXTERNALREF);
								}
							xgrav = xgravity/10;
							ygrav = ygravity/10;
							//printf(" Accelerometer x-axis=%3d\n\r",xgrav);
							//printf(" Accelerometer y-axis=%3d\n\r",ygrav);
							// Reference battery voltage to internal 2.56v ref otherwise
							// input voltage value will not change as battery voltage drops
							//printf(" Battery voltage=%3d\n\r",AnalogCh(2,INTERNALREF));
							//printf(" Temperature sensor=%3d\n\r",AnalogCh(3,EXTERNALREF));
							//printf(" Light sensor=%3d\n\r",AnalogCh(4,EXTERNALREF));
							putcharr('-');
							break;
					case 'v': case 'V':
						    ProgramVersion();
							putcharr('-');
							break;
					case ':':
						    ProgramMemory();
							//putcharr('-');
							break;
					case '1':
					        //printf("l\n\rWrite characters to SmartLCD\n\r");
					        //printf("Hit ESC to exit\n\r");
							while((inchar=getchar())!=ESC) {
							    putchar(inchar);
								putcharSmartLCD(inchar);
								}
							printf("\n\r-");
							break;
					case 'i':
					        putchar('i');
					        DOF10WMenu();
							printf("\n\r-");
							break;
					case 'w':
					        printf("w\n\rClear CRA7 bit");
							lowbyte = Read_HMC5883_Register(HMC5883_CONFIG_REGA);
							Write_HMC5883_Register(HMC5883_CONFIG_REGA,(lowbyte&0x7F));
							printf("\n\r-");
							break;
					case '`':
						    printf("`\n\rSet CRA7 bit");
							Write_HMC5883_Register(HMC5883_CONFIG_REGA,0xf0);
							printf("\n\r-");
							break;
					case '3':
					        printf("3\n\r");	
							Load_BMP085_Parameters();						
							DisplayBMP085('c');
							break;
					case '9':
					        printf("9\n\r");	
							while((inchar=getcharnw())!='e') DisplayDOF10WData();						
							printf("\n\r-");
							break;
					case '4':
					        printf("4\n\r");
							Init_SPI_Master();
							SPI_ADXL345_Write(DATAFORMATADDRESS,0x0B);
							SPI_ADXL345_Write(POWERCONTROLADDRESS,0x08);
							SPI_ADXL345_Write(INTENABLEADDRESS,0x80);
							printf("device id = %x\n\r",SPI_ADXL345_ID());
							Release_SPI();
							break;
					case '5':
						 	printf("5\n\r");
					        DisplayADXL345Data();
							printf("\n\r-");
							break;
//					case 'c': case 'C':
//						    while(!(inchar=getcharnw()))  TestAVR();
//							signon();
//							break;
				    case 't': case 'T':
						    getsettime();
							printf("\n-");
							break;
					case 'g':
						    putcharr('g');
							Display_Gravity_UART(GONLY);
							break;
					case 'r':
						    putcharr('r');
						    numaxis = getchar();
							putchar(numaxis);
							Display_RateGyro_UART(numaxis);
							break;
					case 'e':  case 'E':
							putchar('e');
							FlashChipErase();
							printf("\n\r-");
							break;
					case 'l':  case 'L':
							putchar('l');
							LogDataToSerialFlash();
							printf("\n\r-");
							break;
					case '=':
							//printf("rxcntmax=%d\n-",rxcntmax);
							rxcntmax = 0;
							//printf("totalrxcnt=%d\n-",totalrxcnt);
							rxcntmax = 0;
							//if(frameerror==YES) puts("frame error\n\r");
							//else puts("no frame error\n\r");
							//if(dataoverrun==YES) puts("data overrun\n\r");
							//else puts("no data overrun\n\r");
							//dataoverrun = NO;
							//frameerror = NO;
							printf("xoncnt=%d\n",xoncnt);
							xoncnt = 0;
							xoffcnt = 0;
							break;
					case CR:
							if(flashprogrammingactive==NO)  printf("\n\r-");
							break;
					default:
							putcharr(inchar);
							//Delaymsec(20);
							break;
					}
				}

		
		if((rtccountdown4==0)&&(rtccnt==YES)) {
		    printf(" timeout\n\r");
			rtccnt = NO;
			}
			
		// Blink green led slowly to show AVR is alive
		if(mseccountdown1==0) {
			mseccountdown1 = loopsetting;
			//if(loggingmode==START) mseccountdown1 = loopsetting/4;
			//else mseccountdown1 = loopsetting;
			if(loopmode)  {
			    if(loggingmode==START) GREEN_LED(ON);
				else RED_LED(ON);
				loopmode = 0;
				}
			else {
				if(loggingmode==START) GREEN_LED(OFF);
				else RED_LED(OFF);
				loopmode = 1;
				}
			}
										
		// Clean up flash state machine if no new Intel
		// Hex records arrive for 3 seconds
		if((flashcountdown==0)&&(flashprogrammingactive==YES))
				CleanUpFlashState();
		}
}
